// plugin_alfred_flood.sma
// Author: Alfred Reynolds
// Date September 18th, 2001
//
// Adapted from plugin_sank_message.sma by Sank. 
//
// Fuctions included in this plugin:
// It is a flooding filter so people don't sent too many commands 
// to the server in a short period.
// It determines that a user is flooding the server if they say
// approximately 10 things in less than 2 seconds.
//

#include <core>
#include <console>
#include <string>
#include <admin>
#include <adminlib>

new STRING_VERSION[MAX_DATA_LENGTH] = "2.50.0";

#define MAX_FLOOD 10
#define FLOOD_TIME 2
#define MUTE_TIME 30

// We still don't like spammers, so if they say things too fast, then kick them.
new SndFreq[MAX_PLAYERS][3];			// keep track of sounds per second
#define	FREQ_NUM	0
#define	FREQ_SEC	1
#define FREQ_MUTE	2

/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/


public plugin_init() 
{
	plugin_registerinfo("Anti-Flood Plugin", "mutes flooders", STRING_VERSION);
	
	return PLUGIN_CONTINUE;
}

public plugin_connect(HLUserName, HLIP, UserIndex) 
{
	if (UserIndex >= 1 && UserIndex <= MAX_PLAYERS)
	{
		SndFreq[UserIndex][FREQ_NUM] = 0;
		SndFreq[UserIndex][FREQ_SEC] = 0;
		SndFreq[UserIndex][FREQ_MUTE] = 0;
	}
	return PLUGIN_CONTINUE;
}

public plugin_disconnect(HLUserName, UserIndex) 
{
	if (UserIndex >= 1 && UserIndex <= MAX_PLAYERS)
	{
		SndFreq[UserIndex][FREQ_NUM] = 0;
		SndFreq[UserIndex][FREQ_SEC] = 0;
		SndFreq[UserIndex][FREQ_MUTE] = 0;
	}
	return PLUGIN_CONTINUE;
}

public plugin_command(HLCommand,HLData,HLUserName,UserIndex) 
{
	new User[MAX_NAME_LENGTH];
	convert_string(HLUserName, User, MAX_NAME_LENGTH);

	// If they are talking too fast, have a key bound to say something, or
	// we just think they are flooding the chat then kick them.
	if (FreqCheck(User, UserIndex))
	{
		return PLUGIN_HANDLED;
	}

	return PLUGIN_CONTINUE;

}

/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/


///////////////////////////////////////////////////////////////////////////////
// Returns 0 if the user is allowed to say things
// Returns 1 and kicks the user if the user is saying things too fast.
///////////////////////////////////////////////////////////////////////////////
FreqCheck(User[], UserIndex)
{

	if(check_immunity(User) == 0)
	{
		new CurTime = systemtime();

		// if the user is currently muted
		if(CurTime<= SndFreq[UserIndex][FREQ_MUTE]) return 1;	
	
		// If we're less than one second away from before,
		if (CurTime <= SndFreq[UserIndex][FREQ_SEC] + 1)
		{
			SndFreq[UserIndex][FREQ_NUM] += 1;
			SndFreq[UserIndex][FREQ_SEC] = CurTime;

			if (SndFreq[UserIndex][FREQ_NUM] > MAX_FLOOD)
			{
				// It's even funnier if the clients have this sound loaded,
				// which you can get from the plugin_sank_sounds.sma plugin
				// and put on your server. It also requires the sound.inc file
				// Get them from www.adminmod.org/ascript on the plugins page.

				//playsoundall("misc\comeagain.wav");
				message(User, "You were warned, but did not stop flooding the server. You are now muted for 30 seconds");
				SndFreq[UserIndex][FREQ_MUTE] = CurTime+MUTE_TIME;
				SndFreq[UserIndex][FREQ_NUM] = 0; // reset the command count

				return 1;
			} 
			else if (SndFreq[UserIndex][FREQ_NUM] > MAX_FLOOD/2)
			{
				SndFreq[UserIndex][FREQ_SEC] = CurTime;
				messageex(User, "You are flooding. Stop.", print_center);
				messageex(User, "You are flooding. Stop.", print_console);
				return 0;
			}
		}
		else if (CurTime == (SndFreq[UserIndex][FREQ_SEC] + 2)) // still kinda close
		{

			if (SndFreq[UserIndex][FREQ_NUM] > MAX_FLOOD/2)
			{
				SndFreq[UserIndex][FREQ_SEC] = CurTime;
				messageex(User, "You are flooding. Stop.", print_center);
				messageex(User, "You are flooding. Stop.", print_console);
				return 0;
			}
		}
		else // otherwise we're far enough away that it doesn't matter
		{
			if (SndFreq[UserIndex][FREQ_NUM] != 0)
			{
				// remove X flood records, needs to be largeish
				// because commands usually don't happen often
				// so they build up too quickly if you are not
				// careful.
				SndFreq[UserIndex][FREQ_NUM] -= MAX_FLOOD/2;
			}
			SndFreq[UserIndex][FREQ_SEC] = CurTime;
			return 0;
		}
	}

	return 0;
}
